`regadapt` <-
function(X1,Y1,X2,Y2,d=1,method='poly',model='M2'){
# Usage: reg <- regadapt(X1,Y1,X2,Y2,d=1,method='poly',model='M2');

## Initialization
n2 <- nrow(X2);
p <- ncol(X2);
end <- d * p + 1;		# global dimension of matrices
P <- matrix(0,1,6); 		# for PRESS results
b <- matrix(0,6,end);		# for regressors
bic <- matrix(0,6,end);		# for BIC
aic <- matrix(0,6,end);		# for AIC

## Data transformation
X1 = basismat(X1,d,method);
X2 = basismat(X2,d,method);

## Model estimation
# OLS on P1
b1 = t(inv(t(X1) %*% X1) %*% t(X1) %*% Y1);

# Model M0 (OLS on P2)
if ((model=='M0') || (model=='best')){
	# Compute regressor
	b[1,] = inv(t(X2) %*% X2) %*% t(X2) %*% Y2;
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
		bs = inv(t(X2s) %*% X2s) %*% t(X2s) %*% Y2s;
		P[1] = P[1] + (Y2[i] - t(bs) %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[1,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[1,])^2) / (2*sigma^2);
	bic[1] =  - 2 * ll + (d+1) * log(n2);
	aic[1] = - 2 * ll + 2 * (d+1);
}

# Model M1 ()
if ((model=='M1') || (model=='best')){
	# Compute regressor
# 	Xtmp = X2[,2:end] %*% diag(b1[2:end]);
	Xtmp = X2[,2:end] * matrix(b1[2:end],nrow(X2),end-1,byrow=T);
	lambda = t(inv(t(Xtmp) %*% Xtmp) %*% t(Xtmp) %*% (Y2 - b1[1]));
	b[2,] = c(b1[1],lambda * b1[2:end]);
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
# 		Xtmp = X2s[,2:end] %*% diag(b1[2:end]);
		Xtmp = X2s[,2:end] * matrix(b1[2:end],nrow(X2s),end-1,byrow=T);
		lambda = t(inv(t(Xtmp) %*% Xtmp) %*% t(Xtmp) %*% (Y2s - b1[1]));
		bs = c(b1[1],lambda * b1[2:end]);
		P[2] = P[2] + (Y2[i] - t(bs) %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[2,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[2,])^2) / (2*sigma^2);
	bic[2] =  - 2 * ll + d * log(n2);
	aic[2] =  - 2 * ll + 2 * d;
}

# Model M2 ()
if ((model=='M2') || (model=='best')){
	# Compute regressor
# 	Xtmp = X2 %*% diag(as.vector(b1));
	Xtmp = X2 * matrix(b1,nrow(X2),end,byrow=T);
	Q = matrix(0,nrow(X2),2);
	Q[,1] = Xtmp[,1];
	Q[,2] = rowSums(cbind(Xtmp[,2:end]));
	lambda = t(inv(t(Q) %*% Q) %*% t(Q) %*% Y2);
	b[3,] = c(lambda[1]*b1[1],lambda[2] * b1[2:end]);
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
# 		Xtmp = X2s %*% diag(as.vector(b1));
		Xtmp = X2s * matrix(b1,nrow(X2s),end,byrow=T);
		Q = matrix(0,nrow(X2s),2);
		Q[,1] = Xtmp[,1];
		Q[,2] = rowSums(cbind(Xtmp[,2:end]));
		lambda = t(inv(t(Q) %*% Q) %*% t(Q) %*% Y2s);
		bs = c(lambda[1]*b1[1],lambda[2] * b1[2:end]);
		P[3] = P[3] + (Y2[i] - t(bs) %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[3,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[3,])^2) / (2*sigma^2);
	bic[3] =  - 2 * ll + 2 * log(n2);
	aic[3] =  - 2 * ll + 2 * 2;
}

# Model M3 ()
if ((model=='M3') || (model=='best')){
	# Compute regressor
# 	Xtmp = X2 %*% diag(as.vector(b1));
	Xtmp = X2 * matrix(b1,nrow(X2),end,byrow=T);
	lambda = sum(rowSums(Xtmp)*Y2) / sum(rowSums(Xtmp)^2);
	b[4,] = lambda * b1;
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
# 		Xtmp = X2s %*% diag(as.vector(b1));
		Xtmp = X2s * matrix(b1,nrow(X2s),end,byrow=T);
		lambda = sum(rowSums(Xtmp)*Y2s) / sum(rowSums(Xtmp)^2);
		bs = lambda * b1;
		P[4] = P[4] + (Y2[i] - bs %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[4,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[4,])^2) / (2*sigma^2);
	bic[4] =  - 2 * ll + 1 * log(n2) ;
	aic[4] =  - 2 * ll + 2 * 1;
}

# Model M4 ()
if ((model=='M4') || (model=='best')){
	# Compute regressor
# 	Xtmp = X2[,2:end] %*% diag(b1[,2:end]);
	Xtmp = X2[,2:end] * matrix(b1[2:end],nrow(X2),end-1,byrow=T);
	lambda = sum(rowSums(Xtmp)*(Y2-b1[1])) / sum(rowSums(Xtmp)^2);
	b[5,] = c(b1[1],lambda*b1[2:end]);
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
# 		Xtmp = X2s[,2:end] %*% diag(b1[,2:end]);
		Xtmp = X2s[,2:end] * matrix(b1[2:end],nrow(X2s),end-1,byrow=T);
		lambda = sum(rowSums(Xtmp)*(Y2s-b1[1])) / sum(rowSums(Xtmp)^2);
		bs = c(b1[1],lambda*b1[2:end]);
		P[5] = P[5] + (Y2[i] - bs %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[5,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[5,])^2) / (2*sigma^2);
	bic[5] =  - 2 * ll + 1 * log(n2);
	aic[5] =  - 2 * ll + 2 * 1;
}

# Model M5 ()
if ((model=='M5') || (model=='best')){
	# Compute regressor
# 	Xtmp = X2[,2:end] %*% diag(b1[,2:end]);
	Xtmp = X2[,2:end] * matrix(b1[2:end],nrow(X2),end-1,byrow=T);
	lambda = sum(Y2 - rowSums(Xtmp)) / (nrow(X2) * b1[1]);
	b[6,] = c(lambda*b1[1],b1[2:end]);
	
	# Compute PRESS criterion
	for (i in 1:n2){
		X2s = X2[-i,]; Y2s = Y2[-i,];
# 		Xtmp = X2s[,2:end] %*% diag(b1[,2:end]);
		Xtmp = X2s[,2:end] * matrix(b1[2:end],nrow(X2s),end-1,byrow=T);
		lambda = sum(Y2s - rowSums(Xtmp)) / (nrow(X2s) * b1[1]);
		bs = c(lambda*b1[1],b1[2:end]);
		P[6] = P[6] + (Y2[i] - bs %*% X2[i,])^2;
	}

	# Compute BIC criterion
	sigma = sd(Y2 - X2 %*% b[6,]);
	ll = - n2 * log(sqrt(2*pi)*sigma) - sum((Y2 - X2 %*% b[6,])^2) / (2*sigma^2);
	bic[6] =  - 2 * ll + 1 * log(n2);
	aic[6] =  - 2 * ll + 2 * 1;
}

## Return results
names = c('M0','M1','M2','M3','M4','M5');
P = P / n2; 	#PRESS, BIC and AIC are per point
if (model=='best'){
	P = replace(P,which(P==0),Inf);
	ind = which.min(P);
	reg <- list(model=names[ind],method=method,d=d,prms=b[ind,],press=P[ind],bic=bic[ind]/n2,aic=aic[ind]/n2);
}
else{
	ind = which(names == model);
	reg <- list(model=names[ind],method=method,d=d,prms=b[ind,],press=P[ind],bic=bic[ind]/n2,aic=aic[ind]/n2);
}
reg;
}

